@@ -1,24 +1,28 @@
 # connectors/pyodbc.py
-# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 from . import Connector
-from ..util import asbool
+from .. import util
+
 
 import sys
 import re
-import urllib
 
 
 class PyODBCConnector(Connector):
     driver = 'pyodbc'
 
     supports_sane_multi_rowcount = False
-    # PyODBC unicode is broken on UCS-4 builds
-    supports_unicode = sys.maxunicode == 65535
-    supports_unicode_statements = supports_unicode
+
+    if util.py2k:
+        # PyODBC unicode is broken on UCS-4 builds
+        supports_unicode = sys.maxunicode == 65535
+        supports_unicode_statements = supports_unicode
+
     supports_native_decimal = True
     default_paramstyle = 'named'
 
@@ -56,23 +60,23 @@ class PyODBCConnector(Connector):
         connect_args = {}
         for param in ('ansi', 'unicode_results', 'autocommit'):
             if param in keys:
-                connect_args[param] = asbool(keys.pop(param))
+                connect_args[param] = util.asbool(keys.pop(param))
 
         if 'odbc_connect' in keys:
-            connectors = [urllib.unquote_plus(keys.pop('odbc_connect'))]
+            connectors = [util.unquote_plus(keys.pop('odbc_connect'))]
         else:
             dsn_connection = 'dsn' in keys or \
-                            ('host' in keys and 'database' not in keys)
+                ('host' in keys and 'database' not in keys)
             if dsn_connection:
-                connectors = ['dsn=%s' % (keys.pop('host', '') or \
-                            keys.pop('dsn', ''))]
+                connectors = ['dsn=%s' % (keys.pop('host', '') or
+                                          keys.pop('dsn', ''))]
             else:
                 port = ''
-                if 'port' in keys and not 'port' in query:
+                if 'port' in keys and 'port' not in query:
                     port = ',%d' % int(keys.pop('port'))
 
                 connectors = ["DRIVER={%s}" %
-                                keys.pop('driver', self.pyodbc_driver_name),
+                              keys.pop('driver', self.pyodbc_driver_name),
                               'Server=%s%s' % (keys.pop('host', ''), port),
                               'Database=%s' % keys.pop('database', '')]
 
@@ -89,15 +93,15 @@ class PyODBCConnector(Connector):
             # you query a cp1253 encoded database from a latin1 client...
             if 'odbc_autotranslate' in keys:
                 connectors.append("AutoTranslate=%s" %
-                                    keys.pop("odbc_autotranslate"))
+                                  keys.pop("odbc_autotranslate"))
 
-            connectors.extend(['%s=%s' % (k, v) for k, v in keys.iteritems()])
+            connectors.extend(['%s=%s' % (k, v) for k, v in keys.items()])
         return [[";".join(connectors)], connect_args]
 
     def is_disconnect(self, e, connection, cursor):
         if isinstance(e, self.dbapi.ProgrammingError):
             return "The cursor's connection has been closed." in str(e) or \
-                            'Attempt to use a closed connection.' in str(e)
+                'Attempt to use a closed connection.' in str(e)
         elif isinstance(e, self.dbapi.Error):
             return '[08S01]' in str(e)
         else:
@@ -113,30 +117,32 @@ class PyODBCConnector(Connector):
 
         _sql_driver_name = dbapi_con.getinfo(pyodbc.SQL_DRIVER_NAME)
         self.freetds = bool(re.match(r".*libtdsodbc.*\.so", _sql_driver_name
-                            ))
+                                     ))
         self.easysoft = bool(re.match(r".*libessqlsrv.*\.so", _sql_driver_name
-                            ))
+                                      ))
 
         if self.freetds:
             self.freetds_driver_version = dbapi_con.getinfo(
                 pyodbc.SQL_DRIVER_VER)
 
-        # the "Py2K only" part here is theoretical.
-        # have not tried pyodbc + python3.1 yet.
-        # Py2K
         self.supports_unicode_statements = (
-            not self.freetds and not self.easysoft)
+            not util.py2k or
+            (not self.freetds and not self.easysoft)
+        )
+
         if self._user_supports_unicode_binds is not None:
             self.supports_unicode_binds = self._user_supports_unicode_binds
-        else:
+        elif util.py2k:
             self.supports_unicode_binds = (
                 not self.freetds or self.freetds_driver_version >= '0.91'
             ) and not self.easysoft
-        # end Py2K
+        else:
+            self.supports_unicode_binds = True
 
         # run other initialization which asks for user name, etc.
         super(PyODBCConnector, self).initialize(connection)
 
+
     def _dbapi_version(self):
         if not self.dbapi:
             return ()
@@ -144,9 +150,9 @@ class PyODBCConnector(Connector):
 
     def _parse_dbapi_version(self, vers):
         m = re.match(
-                r'(?:py.*-)?([\d\.]+)(?:-(\w+))?',
-                vers
-            )
+            r'(?:py.*-)?([\d\.]+)(?:-(\w+))?',
+            vers
+        )
         if not m:
             return ()
         vers = tuple([int(x) for x in m.group(1).split(".")])
