@@ -1,5 +1,6 @@
 # sybase/__init__.py
-# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -9,12 +10,12 @@ from sqlalchemy.dialects.sybase import base, pysybase, pyodbc
 # default dialect
 base.dialect = pyodbc.dialect
 
-from base import CHAR, VARCHAR, TIME, NCHAR, NVARCHAR,\
-                 TEXT, DATE, DATETIME, FLOAT, NUMERIC,\
-                 BIGINT, INT, INTEGER, SMALLINT, BINARY,\
-                 VARBINARY, UNITEXT, UNICHAR, UNIVARCHAR,\
-                 IMAGE, BIT, MONEY, SMALLMONEY, TINYINT,\
-                 dialect
+from .base import CHAR, VARCHAR, TIME, NCHAR, NVARCHAR,\
+    TEXT, DATE, DATETIME, FLOAT, NUMERIC,\
+    BIGINT, INT, INTEGER, SMALLINT, BINARY,\
+    VARBINARY, UNITEXT, UNICHAR, UNIVARCHAR,\
+    IMAGE, BIT, MONEY, SMALLMONEY, TINYINT,\
+    dialect
 
 
 __all__ = (
