@@ -1,21 +1,25 @@
 # engine/threadlocal.py
-# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 """Provides a thread-local transactional wrapper around the root Engine class.
 
-The ``threadlocal`` module is invoked when using the ``strategy="threadlocal"`` flag
-with :func:`~sqlalchemy.engine.create_engine`.  This module is semi-private and is 
-invoked automatically when the threadlocal engine strategy is used.
+The ``threadlocal`` module is invoked when using the
+``strategy="threadlocal"`` flag with :func:`~sqlalchemy.engine.create_engine`.
+This module is semi-private and is invoked automatically when the threadlocal
+engine strategy is used.
 """
 
-from sqlalchemy import util
-from sqlalchemy.engine import base
+from .. import util
+from . import base
 import weakref
 
+
 class TLConnection(base.Connection):
+
     def __init__(self, *arg, **kw):
         super(TLConnection, self).__init__(*arg, **kw)
         self.__opencount = 0
@@ -33,19 +37,17 @@ class TLConnection(base.Connection):
         self.__opencount = 0
         base.Connection.close(self)
 
-class TLEngine(base.Engine):
-    """An Engine that includes support for thread-local managed transactions."""
 
+class TLEngine(base.Engine):
+    """An Engine that includes support for thread-local managed
+    transactions.
+
+    """
+    _tl_connection_cls = TLConnection
 
     def __init__(self, *args, **kwargs):
         super(TLEngine, self).__init__(*args, **kwargs)
         self._connections = util.threading.local()
-        proxy = kwargs.get('proxy')
-        if proxy:
-            self.TLConnection = base._proxy_connection_cls(
-                                        TLConnection, proxy)
-        else:
-            self.TLConnection = TLConnection
 
     def contextual_connect(self, **kw):
         if not hasattr(self._connections, 'conn'):
@@ -56,21 +58,24 @@ class TLEngine(base.Engine):
         if connection is None or connection.closed:
             # guards against pool-level reapers, if desired.
             # or not connection.connection.is_valid:
-            connection = self.TLConnection(self, self.pool.connect(), **kw)
-            self._connections.conn = conn = weakref.ref(connection)
+            connection = self._tl_connection_cls(
+                self, self.pool.connect(), **kw)
+            self._connections.conn = weakref.ref(connection)
 
         return connection._increment_connect()
 
     def begin_twophase(self, xid=None):
         if not hasattr(self._connections, 'trans'):
             self._connections.trans = []
-        self._connections.trans.append(self.contextual_connect().begin_twophase(xid=xid))
+        self._connections.trans.append(
+            self.contextual_connect().begin_twophase(xid=xid))
         return self
 
     def begin_nested(self):
         if not hasattr(self._connections, 'trans'):
             self._connections.trans = []
-        self._connections.trans.append(self.contextual_connect().begin_nested())
+        self._connections.trans.append(
+            self.contextual_connect().begin_nested())
         return self
 
     def begin(self):
@@ -90,20 +95,20 @@ class TLEngine(base.Engine):
 
     def prepare(self):
         if not hasattr(self._connections, 'trans') or \
-            not self._connections.trans:
+                not self._connections.trans:
             return
         self._connections.trans[-1].prepare()
 
     def commit(self):
         if not hasattr(self._connections, 'trans') or \
-            not self._connections.trans:
+                not self._connections.trans:
             return
         trans = self._connections.trans.pop(-1)
         trans.commit()
 
     def rollback(self):
         if not hasattr(self._connections, 'trans') or \
-            not self._connections.trans:
+                not self._connections.trans:
             return
         trans = self._connections.trans.pop(-1)
         trans.rollback()
@@ -115,8 +120,8 @@ class TLEngine(base.Engine):
     @property
     def closed(self):
         return not hasattr(self._connections, 'conn') or \
-                self._connections.conn() is None or \
-                self._connections.conn().closed
+            self._connections.conn() is None or \
+            self._connections.conn().closed
 
     def close(self):
         if not self.closed:
