@@ -495,24 +495,33 @@ class Insert(ValuesBase):
          would normally raise an exception if these column lists don't
          correspond.
 
-        .. note::
+        .. warning::
 
-           Depending on backend, it may be necessary for the :class:`.Insert`
-           statement to be constructed using the ``inline=True`` flag; this
-           flag will prevent the implicit usage of ``RETURNING`` when the
-           ``INSERT`` statement is rendered, which isn't supported on a
-           backend such as Oracle in conjunction with an ``INSERT..SELECT``
-           combination::
+           The ``inline=True`` flag should be set to True when using
+           backends that support RETURNING, including Postgresql, Oracle,
+           and SQL Server.   This will prevent the implicit ``RETURNING``
+           clause from being appended to the statement, which is normally
+           used to fetch the "last inserted primary key" value.  This feature
+           will raise an error if the statement inserts zero rows, and
+           on some backends (e.g. Oracle) it will raise an error if the
+           statement inserts more than one row.
+
+           :paramref:`.insert.inline` is set to True as follows::
 
              sel = select([table1.c.a, table1.c.b]).where(table1.c.c > 5)
              ins = table2.insert(inline=True).from_select(['a', 'b'], sel)
 
+           Version 1.0 of SQLAlchemy will set this flag to True in **all**
+           cases.
+
         .. note::
 
-           A SELECT..INSERT construct in SQL has no VALUES clause.  Therefore
-           :class:`.Column` objects which utilize Python-side defaults
-           (e.g. as described at :ref:`metadata_defaults_toplevel`)
-           will **not** take effect when using :meth:`.Insert.from_select`.
+           Python-side and SQL function defaults, as described at
+           :ref:`metadata_defaults_toplevel`, are **not** automatically
+           included in the SELECT statement as rendered unless explicitly
+           added to the statement.  The behavior of automatically rendering
+           these default values and expressions is available as of SQLAlchemy
+           version 1.0.0.
 
         .. versionadded:: 0.8.3
 
@@ -607,12 +616,13 @@ class Update(ValuesBase):
           :meth:`.Update.values` method to set the values of the
           UPDATE statement.
 
-        :param inline:
-          if True, SQL defaults present on :class:`.Column` objects via
-          the ``default`` keyword will be compiled 'inline' into the statement
-          and not pre-executed.  This means that their values will not
-          be available in the dictionary returned from
-          :meth:`.ResultProxy.last_updated_params`.
+        :param inline: if True, no attempt will be made to retrieve the
+            SQL-generated default values to be provided within the statement;
+            in particular,
+            this allows SQL expressions to be rendered 'inline' within the
+            statement without the need to pre-execute them beforehand; for
+            backends that support "returning", this turns off the "implicit
+            returning" feature for the statement.
 
         If both ``values`` and compile-time bind parameters are present, the
         compile-time bind parameters override the information specified
@@ -728,10 +738,10 @@ class Delete(UpdateBase):
         :meth:`~.TableClause.delete` method on
         :class:`~.schema.Table`.
 
-        :param table: The table to be updated.
+        :param table: The table to delete rows from.
 
         :param whereclause: A :class:`.ClauseElement` describing the ``WHERE``
-          condition of the ``UPDATE`` statement. Note that the
+          condition of the ``DELETE`` statement. Note that the
           :meth:`~Delete.where()` generative method may be used instead.
 
         .. seealso::
