@@ -1,8 +1,8 @@
 from sqlalchemy.util import topological
-from test.lib.testing import assert_raises, eq_
-from test.lib.util import conforms_partial_ordering
+from sqlalchemy.testing import assert_raises, eq_
+from sqlalchemy.testing.util import conforms_partial_ordering
 from sqlalchemy import exc
-from test.lib import fixtures
+from sqlalchemy.testing import fixtures
 
 class DependencySortTest(fixtures.TestBase):
 
@@ -82,11 +82,11 @@ class DependencySortTest(fixtures.TestBase):
         try:
             list(topological.sort(tuples, allitems))
             assert False
-        except exc.CircularDependencyError, err:
+        except exc.CircularDependencyError as err:
             eq_(err.cycles, set(['node1', 'node3', 'node2', 'node5',
                 'node4']))
             eq_(err.edges, set([('node3', 'node1'), ('node4', 'node1'),
-                ('node2', 'node3'), ('node1', 'node2'), 
+                ('node2', 'node3'), ('node1', 'node2'),
                 ('node4','node5'), ('node5', 'node4')]))
 
     def test_raise_on_cycle_two(self):
@@ -105,10 +105,10 @@ class DependencySortTest(fixtures.TestBase):
         try:
             list(topological.sort(tuples, allitems))
             assert False
-        except exc.CircularDependencyError, err:
+        except exc.CircularDependencyError as err:
             eq_(err.cycles, set(['node1', 'node3', 'node2']))
             eq_(err.edges, set([('node3', 'node1'), ('node2', 'node3'),
-                ('node3', 'node2'), ('node1', 'node2'), 
+                ('node3', 'node2'), ('node1', 'node2'),
                 ('node2','node4')]))
 
     def test_raise_on_cycle_three(self):
@@ -222,8 +222,11 @@ class DependencySortTest(fixtures.TestBase):
             node5,
             node6,
             ])
-        eq_(topological.find_cycles(tuples, allnodes), set(['node1',
-            'node2', 'node4']))
+        # node6 only became present here once [ticket:2282] was addressed.
+        eq_(
+            topological.find_cycles(tuples, allnodes),
+            set(['node1','node2', 'node4', 'node6'])
+        )
 
     def test_find_multiple_cycles_three(self):
         node1 = 'node1'
@@ -252,3 +255,26 @@ class DependencySortTest(fixtures.TestBase):
             node6,
             ])
         eq_(topological.find_cycles(tuples, allnodes), allnodes)
+
+    def test_find_multiple_cycles_four(self):
+        tuples = [
+            ('node6', 'node2'),
+            ('node15', 'node19'),
+            ('node19', 'node2'), ('node4', 'node10'),
+            ('node15', 'node13'),
+            ('node17', 'node11'), ('node1', 'node19'), ('node15', 'node8'),
+            ('node6', 'node20'), ('node14', 'node11'), ('node6', 'node14'),
+            ('node11', 'node2'), ('node10', 'node20'), ('node1', 'node11'),
+             ('node20', 'node19'), ('node4', 'node20'), ('node15', 'node20'),
+             ('node9', 'node19'), ('node11', 'node10'), ('node11', 'node19'),
+              ('node13', 'node6'), ('node3', 'node15'), ('node9', 'node11'),
+              ('node4', 'node17'), ('node2', 'node20'), ('node19', 'node10'),
+              ('node8', 'node4'), ('node11', 'node3'), ('node6', 'node1')
+        ]
+        allnodes = ['node%d' % i for i in range(1, 21)]
+        eq_(
+            topological.find_cycles(tuples, allnodes),
+            set(['node11', 'node10', 'node13', 'node15', 'node14', 'node17',
+            'node19', 'node20', 'node8', 'node1', 'node3',
+            'node2', 'node4', 'node6'])
+        )
