@@ -1,19 +1,22 @@
-from test.lib.testing import eq_, assert_raises, \
+from sqlalchemy.testing import eq_, assert_raises, \
     assert_raises_message, ne_
 import sys
 import time
 import threading
-from test.lib.engines import testing_engine
+from sqlalchemy import event
+from sqlalchemy.testing.engines import testing_engine
 from sqlalchemy import create_engine, MetaData, INT, VARCHAR, Sequence, \
     select, Integer, String, func, text, exc
-from test.lib.schema import Table
-from test.lib.schema import Column
-from test.lib import testing
-from test.lib import fixtures
+from sqlalchemy.testing.schema import Table
+from sqlalchemy.testing.schema import Column
+from sqlalchemy import testing
+from sqlalchemy.testing import fixtures
 
 
 users, metadata = None, None
 class TransactionTest(fixtures.TestBase):
+    __backend__ = True
+
     @classmethod
     def setup_class(cls):
         global users, metadata
@@ -73,8 +76,8 @@ class TransactionTest(fixtures.TestBase):
             connection.execute(users.insert(), user_id=1, user_name='user3')
             transaction.commit()
             assert False
-        except Exception , e:
-            print "Exception: ", e
+        except Exception as e:
+            print("Exception: ", e)
             transaction.rollback()
 
         result = connection.execute("select * from query_users")
@@ -120,10 +123,10 @@ class TransactionTest(fixtures.TestBase):
                     trans2.rollback()
                     raise
                 transaction.rollback()
-            except Exception, e:
+            except Exception as e:
                 transaction.rollback()
                 raise
-        except Exception, e:
+        except Exception as e:
             try:
                 assert str(e) == 'uh oh'  # and not "This transaction is
                                           # inactive"
@@ -166,7 +169,7 @@ class TransactionTest(fixtures.TestBase):
         connection.execute(users.insert(), user_id=2, user_name='user2')
         try:
             connection.execute(users.insert(), user_id=2, user_name='user2.5')
-        except Exception, e:
+        except Exception as e:
             trans.__exit__(*sys.exc_info())
 
         assert not trans.is_active
@@ -341,7 +344,8 @@ class TransactionTest(fixtures.TestBase):
         transaction = connection.begin_twophase()
         connection.execute(users.insert(), user_id=1, user_name='user1')
         transaction.prepare()
-        connection.close()
+        connection.invalidate()
+
         connection2 = testing.db.connect()
         eq_(connection2.execute(select([users.c.user_id]).
             order_by(users.c.user_id)).fetchall(),
@@ -378,7 +382,142 @@ class TransactionTest(fixtures.TestBase):
         eq_(result.fetchall(), [('user1', ), ('user4', )])
         conn.close()
 
+    @testing.requires.two_phase_transactions
+    def test_reset_rollback_two_phase_no_rollback(self):
+        # test [ticket:2907], essentially that the
+        # TwoPhaseTransaction is given the job of "reset on return"
+        # so that picky backends like MySQL correctly clear out
+        # their state when a connection is closed without handling
+        # the transaction explicitly.
+
+        eng = testing_engine()
+
+        # MySQL raises if you call straight rollback() on
+        # a connection with an XID present
+        @event.listens_for(eng, "invalidate")
+        def conn_invalidated(dbapi_con, con_record, exception):
+            dbapi_con.close()
+            raise exception
+
+        with eng.connect() as conn:
+            rec = conn.connection._connection_record
+            raw_dbapi_con = rec.connection
+            xa = conn.begin_twophase()
+            conn.execute(users.insert(), user_id=1, user_name='user1')
+
+        assert rec.connection is raw_dbapi_con
+
+        with eng.connect() as conn:
+            result = \
+                conn.execute(select([users.c.user_name]).
+                    order_by(users.c.user_id))
+            eq_(result.fetchall(), [])
+
+class ResetAgentTest(fixtures.TestBase):
+    __backend__ = True
+
+    def test_begin_close(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            assert connection.connection._reset_agent is trans
+        assert not trans.is_active
+
+    def test_begin_rollback(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            assert connection.connection._reset_agent is trans
+            trans.rollback()
+            assert connection.connection._reset_agent is None
+
+    def test_begin_commit(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            assert connection.connection._reset_agent is trans
+            trans.commit()
+            assert connection.connection._reset_agent is None
+
+    @testing.requires.savepoints
+    def test_begin_nested_close(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin_nested()
+            assert connection.connection._reset_agent is trans
+        assert not trans.is_active
+
+    @testing.requires.savepoints
+    def test_begin_begin_nested_close(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            trans2 = connection.begin_nested()
+            assert connection.connection._reset_agent is trans
+        assert trans2.is_active  # was never closed
+        assert not trans.is_active
+
+    @testing.requires.savepoints
+    def test_begin_begin_nested_rollback_commit(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            trans2 = connection.begin_nested()
+            assert connection.connection._reset_agent is trans
+            trans2.rollback()
+            assert connection.connection._reset_agent is trans
+            trans.commit()
+            assert connection.connection._reset_agent is None
+
+    @testing.requires.savepoints
+    def test_begin_begin_nested_rollback_rollback(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            trans2 = connection.begin_nested()
+            assert connection.connection._reset_agent is trans
+            trans2.rollback()
+            assert connection.connection._reset_agent is trans
+            trans.rollback()
+            assert connection.connection._reset_agent is None
+
+    def test_begin_begin_rollback_rollback(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            trans2 = connection.begin()
+            assert connection.connection._reset_agent is trans
+            trans2.rollback()
+            assert connection.connection._reset_agent is None
+            trans.rollback()
+            assert connection.connection._reset_agent is None
+
+    def test_begin_begin_commit_commit(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin()
+            trans2 = connection.begin()
+            assert connection.connection._reset_agent is trans
+            trans2.commit()
+            assert connection.connection._reset_agent is trans
+            trans.commit()
+            assert connection.connection._reset_agent is None
+
+    @testing.requires.two_phase_transactions
+    def test_reset_via_agent_begin_twophase(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin_twophase()
+            assert connection.connection._reset_agent is trans
+
+    @testing.requires.two_phase_transactions
+    def test_reset_via_agent_begin_twophase_commit(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin_twophase()
+            assert connection.connection._reset_agent is trans
+            trans.commit()
+            assert connection.connection._reset_agent is None
+
+    @testing.requires.two_phase_transactions
+    def test_reset_via_agent_begin_twophase_rollback(self):
+        with testing.db.connect() as connection:
+            trans = connection.begin_twophase()
+            assert connection.connection._reset_agent is trans
+            trans.rollback()
+            assert connection.connection._reset_agent is None
+
 class AutoRollbackTest(fixtures.TestBase):
+    __backend__ = True
 
     @classmethod
     def setup_class(cls):
@@ -503,7 +642,7 @@ class ExplicitAutoCommitTest(fixtures.TestBase):
         conn2.close()
 
     @testing.uses_deprecated(r'autocommit on select\(\) is deprecated',
-                             r'autocommit\(\) is deprecated')
+                             r'``autocommit\(\)`` is deprecated')
     def test_explicit_compiled_deprecated(self):
         conn1 = testing.db.connect()
         conn2 = testing.db.connect()
@@ -544,6 +683,7 @@ tlengine = None
 
 class TLTransactionTest(fixtures.TestBase):
     __requires__ = ('ad_hoc_engines', )
+    __backend__ = True
 
     @classmethod
     def setup_class(cls):
@@ -816,7 +956,7 @@ class TLTransactionTest(fixtures.TestBase):
 
     def test_mixed_nesting(self):
         """tests nesting of transactions off the TLEngine directly
-        inside of tranasctions off the connection from the TLEngine"""
+        inside of transactions off the connection from the TLEngine"""
 
         external_connection = tlengine.connect()
         self.assert_(external_connection.connection
@@ -848,7 +988,7 @@ class TLTransactionTest(fixtures.TestBase):
 
     def test_more_mixed_nesting(self):
         """tests nesting of transactions off the connection from the
-        TLEngine inside of tranasctions off thbe TLEngine directly."""
+        TLEngine inside of transactions off the TLEngine directly."""
 
         external_connection = tlengine.connect()
         self.assert_(external_connection.connection
@@ -985,163 +1125,10 @@ class TLTransactionTest(fixtures.TestBase):
             order_by(users.c.user_id)).fetchall(),
             [(1, ), (2, )])
 
-counters = None
-
-
-class ForUpdateTest(fixtures.TestBase):
-    __requires__ = 'ad_hoc_engines',
-
-    @classmethod
-    def setup_class(cls):
-        global counters, metadata
-        metadata = MetaData()
-        counters = Table('forupdate_counters', metadata,
-                         Column('counter_id', INT, primary_key=True),
-                         Column('counter_value', INT),
-                         test_needs_acid=True)
-        counters.create(testing.db)
-
-    def teardown(self):
-        testing.db.execute(counters.delete()).close()
-
-    @classmethod
-    def teardown_class(cls):
-        counters.drop(testing.db)
-
-    def increment(
-        self,
-        count,
-        errors,
-        update_style=True,
-        delay=0.005,
-        ):
-        con = testing.db.connect()
-        sel = counters.select(for_update=update_style,
-                              whereclause=counters.c.counter_id == 1)
-        for i in xrange(count):
-            trans = con.begin()
-            try:
-                existing = con.execute(sel).first()
-                incr = existing['counter_value'] + 1
-                time.sleep(delay)
-                con.execute(counters.update(counters.c.counter_id == 1,
-                            values={'counter_value': incr}))
-                time.sleep(delay)
-                readback = con.execute(sel).first()
-                if readback['counter_value'] != incr:
-                    raise AssertionError('Got %s post-update, expected '
-                            '%s' % (readback['counter_value'], incr))
-                trans.commit()
-            except Exception, e:
-                trans.rollback()
-                errors.append(e)
-                break
-        con.close()
-
-    @testing.crashes('mssql', 'FIXME: unknown')
-    @testing.crashes('firebird', 'FIXME: unknown')
-    @testing.crashes('sybase', 'FIXME: unknown')
-    @testing.crashes('access', 'FIXME: unknown')
-    @testing.requires.independent_connections
-    def test_queued_update(self):
-        """Test SELECT FOR UPDATE with concurrent modifications.
-
-        Runs concurrent modifications on a single row in the users
-        table, with each mutator trying to increment a value stored in
-        user_name.
-
-        """
-
-        db = testing.db
-        db.execute(counters.insert(), counter_id=1, counter_value=0)
-        iterations, thread_count = 10, 5
-        threads, errors = [], []
-        for i in xrange(thread_count):
-            thrd = threading.Thread(target=self.increment,
-                                    args=(iterations, ),
-                                    kwargs={'errors': errors,
-                                    'update_style': True})
-            thrd.start()
-            threads.append(thrd)
-        for thrd in threads:
-            thrd.join()
-        for e in errors:
-            sys.stdout.write('Failure: %s\n' % e)
-        self.assert_(len(errors) == 0)
-        sel = counters.select(whereclause=counters.c.counter_id == 1)
-        final = db.execute(sel).first()
-        self.assert_(final['counter_value'] == iterations
-                     * thread_count)
-
-    def overlap(
-        self,
-        ids,
-        errors,
-        update_style,
-        ):
-        sel = counters.select(for_update=update_style,
-                              whereclause=counters.c.counter_id.in_(ids))
-        con = testing.db.connect()
-        trans = con.begin()
-        try:
-            rows = con.execute(sel).fetchall()
-            time.sleep(0.25)
-            trans.commit()
-        except Exception, e:
-            trans.rollback()
-            errors.append(e)
-        con.close()
-
-    def _threaded_overlap(
-        self,
-        thread_count,
-        groups,
-        update_style=True,
-        pool=5,
-        ):
-        db = testing.db
-        for cid in range(pool - 1):
-            db.execute(counters.insert(), counter_id=cid + 1,
-                       counter_value=0)
-        errors, threads = [], []
-        for i in xrange(thread_count):
-            thrd = threading.Thread(target=self.overlap,
-                                    args=(groups.pop(0), errors,
-                                    update_style))
-            thrd.start()
-            threads.append(thrd)
-        for thrd in threads:
-            thrd.join()
-        return errors
-
-    @testing.crashes('mssql', 'FIXME: unknown')
-    @testing.crashes('firebird', 'FIXME: unknown')
-    @testing.crashes('sybase', 'FIXME: unknown')
-    @testing.crashes('access', 'FIXME: unknown')
-    @testing.requires.independent_connections
-    def test_queued_select(self):
-        """Simple SELECT FOR UPDATE conflict test"""
-
-        errors = self._threaded_overlap(2, [(1, 2, 3), (3, 4, 5)])
-        for e in errors:
-            sys.stderr.write('Failure: %s\n' % e)
-        self.assert_(len(errors) == 0)
-
-    @testing.crashes('mssql', 'FIXME: unknown')
-    @testing.fails_on('mysql', 'No support for NOWAIT')
-    @testing.crashes('firebird', 'FIXME: unknown')
-    @testing.crashes('sybase', 'FIXME: unknown')
-    @testing.crashes('access', 'FIXME: unknown')
-    @testing.requires.independent_connections
-    def test_nowait_select(self):
-        """Simple SELECT FOR UPDATE NOWAIT conflict test"""
-
-        errors = self._threaded_overlap(2, [(1, 2, 3), (3, 4, 5)],
-                update_style='nowait')
-        self.assert_(len(errors) != 0)
 
 class IsolationLevelTest(fixtures.TestBase):
     __requires__ = ('isolation_level', 'ad_hoc_engines')
+    __backend__ = True
 
     def _default_isolation_level(self):
         if testing.against('sqlite'):
@@ -1297,15 +1284,15 @@ class IsolationLevelTest(fixtures.TestBase):
         )
 
 
-    def test_per_engine_bzzt(self):
-        assert_raises_message(
-            exc.ArgumentError,
-            r"'isolation_level' execution option may "
-            r"only be specified on Connection.execution_options\(\). "
-            r"To set engine-wide isolation level, "
-            r"use the isolation_level argument to create_engine\(\).",
-            create_engine,
-            testing.db.url,
-                execution_options={'isolation_level':
-                            self._non_default_isolation_level}
+    def test_per_engine(self):
+        # new in 0.9
+        eng = create_engine(testing.db.url,
+                            execution_options={
+                                'isolation_level':
+                                    self._non_default_isolation_level()}
+                        )
+        conn = eng.connect()
+        eq_(
+            eng.dialect.get_isolation_level(conn.connection),
+            self._non_default_isolation_level()
         )
