@@ -1,18 +1,25 @@
-from test.lib.testing import eq_, assert_raises, assert_raises_message
+from sqlalchemy.testing import eq_, assert_raises_message, assert_raises
 import datetime
 from sqlalchemy.schema import CreateSequence, DropSequence
-from sqlalchemy.sql import select, text, literal_column
+from sqlalchemy.sql import select, text
 import sqlalchemy as sa
-from test.lib import testing, engines
-from sqlalchemy import MetaData, Integer, String, ForeignKey, Boolean, exc,\
-                Sequence, func, literal, Unicode
+from sqlalchemy import testing
+from sqlalchemy.testing import engines
+from sqlalchemy import (
+    MetaData, Integer, String, ForeignKey, Boolean, exc, Sequence, func,
+    literal, Unicode, cast)
 from sqlalchemy.types import TypeDecorator, TypeEngine
-from test.lib.schema import Table, Column
-from test.lib.testing import eq_
+from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.dialects import sqlite
-from test.lib import fixtures
+from sqlalchemy.testing import fixtures
+from sqlalchemy.util import u, b
+from sqlalchemy import util
+
+t = f = f2 = ts = currenttime = metadata = default_generator = None
+
 
 class DefaultTest(fixtures.TestBase):
+    __backend__ = True
 
     @classmethod
     def setup_class(cls):
@@ -20,7 +27,7 @@ class DefaultTest(fixtures.TestBase):
 
         db = testing.db
         metadata = MetaData(db)
-        default_generator = {'x':50}
+        default_generator = {'x': 50}
 
         def mydefault():
             default_generator['x'] += 1
@@ -39,22 +46,37 @@ class DefaultTest(fixtures.TestBase):
                 # since its a "branched" connection
                 conn.close()
 
-        use_function_defaults = testing.against('postgresql', 'mssql', 'maxdb')
+        use_function_defaults = testing.against('postgresql', 'mssql')
         is_oracle = testing.against('oracle')
 
+        class MyClass(object):
+
+            @classmethod
+            def gen_default(cls, ctx):
+                return "hi"
+
         # select "count(1)" returns different results on different DBs also
         # correct for "current_date" compatible as column default, value
         # differences
         currenttime = func.current_date(type_=sa.Date, bind=db)
         if is_oracle:
-            ts = db.scalar(sa.select([func.trunc(func.sysdate(), sa.literal_column("'DAY'"), type_=sa.Date).label('today')]))
-            assert isinstance(ts, datetime.date) and not isinstance(ts, datetime.datetime)
+            ts = db.scalar(
+                sa.select(
+                    [
+                        func.trunc(
+                            func.sysdate(), sa.literal_column("'DAY'"),
+                            type_=sa.Date).label('today')]))
+            assert isinstance(ts, datetime.date) and not isinstance(
+                ts, datetime.datetime)
             f = sa.select([func.length('abcdef')], bind=db).scalar()
             f2 = sa.select([func.length('abcdefghijk')], bind=db).scalar()
             # TODO: engine propigation across nested functions not working
-            currenttime = func.trunc(currenttime, sa.literal_column("'DAY'"), bind=db, type_=sa.Date)
+            currenttime = func.trunc(
+                currenttime, sa.literal_column("'DAY'"), bind=db,
+                type_=sa.Date)
             def1 = currenttime
-            def2 = func.trunc(sa.text("sysdate"), sa.literal_column("'DAY'"), type_=sa.Date)
+            def2 = func.trunc(
+                sa.text("sysdate"), sa.literal_column("'DAY'"), type_=sa.Date)
 
             deftype = sa.Date
         elif use_function_defaults:
@@ -62,9 +84,7 @@ class DefaultTest(fixtures.TestBase):
             f2 = sa.select([func.length('abcdefghijk')], bind=db).scalar()
             def1 = currenttime
             deftype = sa.Date
-            if testing.against('maxdb'):
-                def2 = sa.text("curdate")
-            elif testing.against('mssql'):
+            if testing.against('mssql'):
                 def2 = sa.text("getdate()")
             else:
                 def2 = sa.text("current_date")
@@ -76,7 +96,8 @@ class DefaultTest(fixtures.TestBase):
             ts = 3
             deftype = Integer
 
-        t = Table('default_test1', metadata,
+        t = Table(
+            'default_test1', metadata,
             # python function
             Column('col1', Integer, primary_key=True,
                    default=mydefault),
@@ -119,7 +140,12 @@ class DefaultTest(fixtures.TestBase):
             # combo
             Column('col9', String(20),
                    default='py',
-                   server_default='ddl'))
+                   server_default='ddl'),
+
+            # python method w/ context
+            Column('col10', String(20), default=MyClass.gen_default)
+        )
+
         t.create()
 
     @classmethod
@@ -131,47 +157,70 @@ class DefaultTest(fixtures.TestBase):
         t.delete().execute()
 
     def test_bad_arg_signature(self):
-        ex_msg = \
-          "ColumnDefault Python function takes zero or one positional arguments"
+        ex_msg = "ColumnDefault Python function takes zero " \
+            "or one positional arguments"
+
+        def fn1(x, y):
+            pass
+
+        def fn2(x, y, z=3):
+            pass
 
-        def fn1(x, y): pass
-        def fn2(x, y, z=3): pass
         class fn3(object):
+
             def __init__(self, x, y):
                 pass
+
         class FN4(object):
+
             def __call__(self, x, y):
                 pass
         fn4 = FN4()
 
         for fn in fn1, fn2, fn3, fn4:
-            assert_raises_message(sa.exc.ArgumentError,
-                                     ex_msg,
-                                     sa.ColumnDefault, fn)
+            assert_raises_message(
+                sa.exc.ArgumentError, ex_msg, sa.ColumnDefault, fn)
 
     def test_arg_signature(self):
-        def fn1(): pass
-        def fn2(): pass
-        def fn3(x=1): pass
-        def fn4(x=1, y=2, z=3): pass
+
+        def fn1():
+            pass
+
+        def fn2():
+            pass
+
+        def fn3(x=1):
+            eq_(x, 1)
+
+        def fn4(x=1, y=2, z=3):
+            eq_(x, 1)
         fn5 = list
-        class fn6(object):
+
+        class fn6a(object):
+
             def __init__(self, x):
-                pass
-        class fn6(object):
+                eq_(x, "context")
+
+        class fn6b(object):
+
             def __init__(self, x, y=3):
-                pass
+                eq_(x, "context")
+
         class FN7(object):
+
             def __call__(self, x):
-                pass
+                eq_(x, "context")
         fn7 = FN7()
+
         class FN8(object):
+
             def __call__(self, x, y=3):
-                pass
+                eq_(x, "context")
         fn8 = FN8()
 
-        for fn in fn1, fn2, fn3, fn4, fn5, fn6, fn7, fn8:
+        for fn in fn1, fn2, fn3, fn4, fn5, fn6a, fn6b, fn7, fn8:
             c = sa.ColumnDefault(fn)
+            c.arg("context")
 
     @testing.fails_on('firebird', 'Data type unknown')
     def test_standalone(self):
@@ -187,7 +236,8 @@ class DefaultTest(fixtures.TestBase):
     def test_py_vs_server_default_detection(self):
 
         def has_(name, *wanted):
-            slots = ['default', 'onupdate', 'server_default', 'server_onupdate']
+            slots = [
+                'default', 'onupdate', 'server_default', 'server_onupdate']
             col = tbl.c[name]
             for slot in wanted:
                 slots.remove(slot)
@@ -250,7 +300,8 @@ class DefaultTest(fixtures.TestBase):
         has_('col5', 'default', 'server_default', 'onupdate')
         has_('col6', 'default', 'server_default', 'onupdate')
         has_('col7', 'default', 'server_default', 'onupdate')
-        has_('col8', 'default', 'server_default', 'onupdate', 'server_onupdate')
+        has_(
+            'col8', 'default', 'server_default', 'onupdate', 'server_onupdate')
 
     @testing.fails_on('firebird', 'Data type unknown')
     def test_insert(self):
@@ -266,12 +317,13 @@ class DefaultTest(fixtures.TestBase):
 
         t.insert().execute()
 
-        ctexec = sa.select([currenttime.label('now')], bind=testing.db).scalar()
+        ctexec = sa.select(
+            [currenttime.label('now')], bind=testing.db).scalar()
         l = t.select().order_by(t.c.col1).execute()
         today = datetime.date.today()
         eq_(l.fetchall(), [
             (x, 'imthedefault', f, ts, ts, ctexec, True, False,
-             12, today, 'py')
+             12, today, 'py', 'hi')
             for x in range(51, 54)])
 
         t.insert().execute(col9=None)
@@ -279,29 +331,29 @@ class DefaultTest(fixtures.TestBase):
         eq_(set(r.context.postfetch_cols),
             set([t.c.col3, t.c.col5, t.c.col4, t.c.col6]))
 
-        eq_(t.select(t.c.col1==54).execute().fetchall(),
+        eq_(t.select(t.c.col1 == 54).execute().fetchall(),
             [(54, 'imthedefault', f, ts, ts, ctexec, True, False,
-              12, today, None)])
+              12, today, None, 'hi')])
 
     @testing.fails_on('firebird', 'Data type unknown')
     def test_insertmany(self):
         # MySQL-Python 1.2.2 breaks functions in execute_many :(
         if (testing.against('mysql+mysqldb') and
-            testing.db.dialect.dbapi.version_info[:3] == (1, 2, 2)):
+                testing.db.dialect.dbapi.version_info[:3] == (1, 2, 2)):
             return
 
-        r = t.insert().execute({}, {}, {})
+        t.insert().execute({}, {}, {})
 
         ctexec = currenttime.scalar()
         l = t.select().execute()
         today = datetime.date.today()
         eq_(l.fetchall(),
             [(51, 'imthedefault', f, ts, ts, ctexec, True, False,
-              12, today, 'py'),
+              12, today, 'py', 'hi'),
              (52, 'imthedefault', f, ts, ts, ctexec, True, False,
-              12, today, 'py'),
+              12, today, 'py', 'hi'),
              (53, 'imthedefault', f, ts, ts, ctexec, True, False,
-              12, today, 'py')])
+              12, today, 'py', 'hi')])
 
     def test_no_embed_in_sql(self):
         """Using a DefaultGenerator, Sequence, DefaultClause
@@ -330,16 +382,18 @@ class DefaultTest(fixtures.TestBase):
             )
 
     def test_missing_many_param(self):
-        assert_raises_message(exc.StatementError,
-            "A value is required for bind parameter 'col7', in parameter group 1",
+        assert_raises_message(
+            exc.StatementError,
+            "A value is required for bind parameter 'col7', in parameter "
+            "group 1",
             t.insert().execute,
-            {'col4':7, 'col7':12, 'col8':19},
-            {'col4':7, 'col8':19},
-            {'col4':7, 'col7':12, 'col8':19},
+            {'col4': 7, 'col7': 12, 'col8': 19},
+            {'col4': 7, 'col8': 19},
+            {'col4': 7, 'col7': 12, 'col8': 19},
         )
 
     def test_insert_values(self):
-        t.insert(values={'col3':50}).execute()
+        t.insert(values={'col3': 50}).execute()
         l = t.select().execute()
         eq_(50, l.first()['col3'])
 
@@ -347,65 +401,69 @@ class DefaultTest(fixtures.TestBase):
     def test_updatemany(self):
         # MySQL-Python 1.2.2 breaks functions in execute_many :(
         if (testing.against('mysql+mysqldb') and
-            testing.db.dialect.dbapi.version_info[:3] == (1, 2, 2)):
+                testing.db.dialect.dbapi.version_info[:3] == (1, 2, 2)):
             return
 
         t.insert().execute({}, {}, {})
 
-        t.update(t.c.col1==sa.bindparam('pkval')).execute(
-            {'pkval':51,'col7':None, 'col8':None, 'boolcol1':False})
+        t.update(t.c.col1 == sa.bindparam('pkval')).execute(
+            {'pkval': 51, 'col7': None, 'col8': None, 'boolcol1': False})
 
-        t.update(t.c.col1==sa.bindparam('pkval')).execute(
-            {'pkval':51,},
-            {'pkval':52,},
-            {'pkval':53,})
+        t.update(t.c.col1 == sa.bindparam('pkval')).execute(
+            {'pkval': 51},
+            {'pkval': 52},
+            {'pkval': 53})
 
         l = t.select().execute()
         ctexec = currenttime.scalar()
         today = datetime.date.today()
         eq_(l.fetchall(),
             [(51, 'im the update', f2, ts, ts, ctexec, False, False,
-              13, today, 'py'),
+              13, today, 'py', 'hi'),
              (52, 'im the update', f2, ts, ts, ctexec, True, False,
-              13, today, 'py'),
+              13, today, 'py', 'hi'),
              (53, 'im the update', f2, ts, ts, ctexec, True, False,
-              13, today, 'py')])
+              13, today, 'py', 'hi')])
 
     @testing.fails_on('firebird', 'Data type unknown')
     def test_update(self):
         r = t.insert().execute()
         pk = r.inserted_primary_key[0]
-        t.update(t.c.col1==pk).execute(col4=None, col5=None)
+        t.update(t.c.col1 == pk).execute(col4=None, col5=None)
         ctexec = currenttime.scalar()
-        l = t.select(t.c.col1==pk).execute()
+        l = t.select(t.c.col1 == pk).execute()
         l = l.first()
         eq_(l,
             (pk, 'im the update', f2, None, None, ctexec, True, False,
-             13, datetime.date.today(), 'py'))
+             13, datetime.date.today(), 'py', 'hi'))
         eq_(11, f2)
 
     @testing.fails_on('firebird', 'Data type unknown')
     def test_update_values(self):
         r = t.insert().execute()
         pk = r.inserted_primary_key[0]
-        t.update(t.c.col1==pk, values={'col3': 55}).execute()
-        l = t.select(t.c.col1==pk).execute()
+        t.update(t.c.col1 == pk, values={'col3': 55}).execute()
+        l = t.select(t.c.col1 == pk).execute()
         l = l.first()
         eq_(55, l['col3'])
 
 
 class PKDefaultTest(fixtures.TablesTest):
     __requires__ = ('subqueries',)
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
-        t2 = Table('t2', metadata,
+        t2 = Table(
+            't2', metadata,
             Column('nextid', Integer))
 
-        Table('t1', metadata,
-              Column('id', Integer, primary_key=True,
-                     default=sa.select([func.max(t2.c.nextid)]).as_scalar()),
-              Column('data', String(30)))
+        Table(
+            't1', metadata,
+            Column(
+                'id', Integer, primary_key=True,
+                default=sa.select([func.max(t2.c.nextid)]).as_scalar()),
+            Column('data', String(30)))
 
     @testing.requires.returning
     def test_with_implicit_returning(self):
@@ -420,7 +478,8 @@ class PKDefaultTest(fixtures.TablesTest):
         if not returning and not testing.db.dialect.implicit_returning:
             engine = testing.db
         else:
-            engine = engines.testing_engine(options={'implicit_returning':returning})
+            engine = engines.testing_engine(
+                options={'implicit_returning': returning})
         engine.execute(t2.insert(), nextid=1)
         r = engine.execute(t1.insert(), data='hi')
         eq_([1], r.inserted_primary_key)
@@ -429,8 +488,10 @@ class PKDefaultTest(fixtures.TablesTest):
         r = engine.execute(t1.insert(), data='there')
         eq_([2], r.inserted_primary_key)
 
+
 class PKIncrementTest(fixtures.TablesTest):
     run_define_tables = 'each'
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
@@ -464,13 +525,13 @@ class PKIncrementTest(fixtures.TablesTest):
         self.assert_(last not in ids)
         ids.add(last)
 
-        rs = bind.execute(aitable.insert(values={'int1':func.length('four')}))
+        rs = bind.execute(aitable.insert(values={'int1': func.length('four')}))
         last = rs.inserted_primary_key[0]
         self.assert_(last)
         self.assert_(last not in ids)
         ids.add(last)
 
-        eq_(ids, set([1,2,3,4]))
+        eq_(ids, set([1, 2, 3, 4]))
 
         eq_(list(bind.execute(aitable.select().order_by(aitable.c.id))),
             [(1, 1, None), (2, None, 'row 2'), (3, 3, 'row 3'), (4, 4, None)])
@@ -497,24 +558,25 @@ class PKIncrementTest(fixtures.TablesTest):
 
 
 class EmptyInsertTest(fixtures.TestBase):
+    __backend__ = True
+
     @testing.exclude('sqlite', '<', (3, 3, 8), 'no empty insert support')
     @testing.fails_on('oracle', 'FIXME: unknown')
+    @testing.provide_metadata
     def test_empty_insert(self):
-        metadata = MetaData(testing.db)
-        t1 = Table('t1', metadata,
-                Column('is_true', Boolean, server_default=('1')))
-        metadata.create_all()
+        t1 = Table(
+            't1', self.metadata,
+            Column('is_true', Boolean, server_default=('1')))
+        self.metadata.create_all()
+        t1.insert().execute()
+        eq_(1, select([func.count(text('*'))], from_obj=t1).scalar())
+        eq_(True, t1.select().scalar())
 
-        try:
-            result = t1.insert().execute()
-            eq_(1, select([func.count(text('*'))], from_obj=t1).scalar())
-            eq_(True, t1.select().scalar())
-        finally:
-            metadata.drop_all()
 
 class AutoIncrementTest(fixtures.TablesTest):
     __requires__ = ('identity',)
     run_define_tables = 'each'
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
@@ -532,7 +594,8 @@ class AutoIncrementTest(fixtures.TablesTest):
         eq_(1, sa.select([func.count(sa.text('*'))], from_obj=single).scalar())
 
     def test_autoincrement_fk(self):
-        nodes = Table('nodes', self.metadata,
+        nodes = Table(
+            'nodes', self.metadata,
             Column('id', Integer, primary_key=True),
             Column('parent_id', Integer, ForeignKey('nodes.id')),
             Column('data', String(30)))
@@ -547,57 +610,87 @@ class AutoIncrementTest(fixtures.TablesTest):
             impl = TypeEngine
 
         assert MyType()._type_affinity is None
-        t = Table('x', MetaData(),
+        t = Table(
+            'x', MetaData(),
             Column('id', MyType(), primary_key=True)
         )
         assert t._autoincrement_column is None
 
     def test_autoincrement_ignore_fk(self):
         m = MetaData()
-        y = Table('y', m,
+        Table(
+            'y', m,
             Column('id', Integer(), primary_key=True)
         )
-        x = Table('x', m,
-            Column('id', Integer(),
-                ForeignKey('y.id'),
+        x = Table(
+            'x', m,
+            Column(
+                'id', Integer(), ForeignKey('y.id'),
                 autoincrement="ignore_fk", primary_key=True)
         )
         assert x._autoincrement_column is x.c.id
 
     def test_autoincrement_fk_disqualifies(self):
         m = MetaData()
-        y = Table('y', m,
+        Table(
+            'y', m,
             Column('id', Integer(), primary_key=True)
         )
-        x = Table('x', m,
-            Column('id', Integer(),
-                ForeignKey('y.id'),
-                primary_key=True)
+        x = Table(
+            'x', m,
+            Column('id', Integer(), ForeignKey('y.id'), primary_key=True)
         )
         assert x._autoincrement_column is None
 
     @testing.fails_on('sqlite', 'FIXME: unknown')
     def test_non_autoincrement(self):
         # sqlite INT primary keys can be non-unique! (only for ints)
-        nonai = Table("nonaitest", self.metadata,
+        nonai = Table(
+            "nonaitest", self.metadata,
             Column('id', Integer, autoincrement=False, primary_key=True),
             Column('data', String(20)))
         nonai.create()
 
-
-        try:
+        def go():
             # postgresql + mysql strict will fail on first row,
             # mysql in legacy mode fails on second row
             nonai.insert().execute(data='row 1')
             nonai.insert().execute(data='row 2')
-            assert False
-        except sa.exc.DBAPIError, e:
-            assert True
+        assert_raises(
+            sa.exc.DBAPIError,
+            go
+        )
 
         nonai.insert().execute(id=1, data='row 1')
 
+    def test_col_w_sequence_non_autoinc_no_firing(self):
+        metadata = self.metadata
+        # plain autoincrement/PK table in the actual schema
+        Table(
+            "x", metadata,
+            Column("set_id", Integer, primary_key=True)
+        )
+        metadata.create_all()
+
+        # for the INSERT use a table with a Sequence
+        # and autoincrement=False.  Using a ForeignKey
+        # would have the same effect
+        dataset_no_autoinc = Table(
+            "x", MetaData(),
+            Column(
+                "set_id", Integer, Sequence("some_seq"),
+                primary_key=True, autoincrement=False)
+        )
+
+        testing.db.execute(dataset_no_autoinc.insert())
+        eq_(
+            testing.db.scalar(dataset_no_autoinc.count()), 1
+        )
+
+
 class SequenceDDLTest(fixtures.TestBase, testing.AssertsCompiledSQL):
     __dialect__ = 'default'
+    __backend__ = True
 
     def test_create_drop_ddl(self):
         self.assert_compile(
@@ -625,12 +718,14 @@ class SequenceDDLTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             "DROP SEQUENCE foo_seq",
         )
 
+
 class SequenceExecTest(fixtures.TestBase):
     __requires__ = ('sequences',)
+    __backend__ = True
 
     @classmethod
     def setup_class(cls):
-        cls.seq= Sequence("my_sequence")
+        cls.seq = Sequence("my_sequence")
         cls.seq.create(testing.db)
 
     @classmethod
@@ -640,7 +735,7 @@ class SequenceExecTest(fixtures.TestBase):
     def _assert_seq_result(self, ret):
         """asserts return of next_value is an int"""
 
-        assert isinstance(ret, (int, long))
+        assert isinstance(ret, util.int_types)
         assert ret > 0
 
     def test_implicit_connectionless(self):
@@ -689,11 +784,12 @@ class SequenceExecTest(fixtures.TestBase):
         """test can use next_value() in whereclause"""
 
         metadata = self.metadata
-        t1 = Table('t', metadata,
+        t1 = Table(
+            't', metadata,
             Column('x', Integer)
         )
         t1.create(testing.db)
-        testing.db.execute(t1.insert(), [{'x':1}, {'x':300}, {'x':301}])
+        testing.db.execute(t1.insert(), [{'x': 1}, {'x': 300}, {'x': 301}])
         s = Sequence("my_sequence")
         eq_(
             testing.db.execute(
@@ -707,7 +803,8 @@ class SequenceExecTest(fixtures.TestBase):
         """test can use next_value() in values() of _ValuesBase"""
 
         metadata = self.metadata
-        t1 = Table('t', metadata,
+        t1 = Table(
+            't', metadata,
             Column('x', Integer)
         )
         t1.create(testing.db)
@@ -725,16 +822,15 @@ class SequenceExecTest(fixtures.TestBase):
         pk_col=next_value(), implicit returning is not used."""
 
         metadata = self.metadata
-        e = engines.testing_engine(options={'implicit_returning':False})
+        e = engines.testing_engine(options={'implicit_returning': False})
         s = Sequence("my_sequence")
         metadata.bind = e
-        t1 = Table('t', metadata,
+        t1 = Table(
+            't', metadata,
             Column('x', Integer, primary_key=True)
         )
         t1.create()
-        r = e.execute(
-            t1.insert().values(x=s.next_value())
-        )
+        r = e.execute(t1.insert().values(x=s.next_value()))
         eq_(r.inserted_primary_key, [None])
 
     @testing.requires.returning
@@ -744,10 +840,11 @@ class SequenceExecTest(fixtures.TestBase):
         pk_col=next_value(), when implicit returning is used."""
 
         metadata = self.metadata
-        e = engines.testing_engine(options={'implicit_returning':True})
+        e = engines.testing_engine(options={'implicit_returning': True})
         s = Sequence("my_sequence")
         metadata.bind = e
-        t1 = Table('t', metadata,
+        t1 = Table(
+            't', metadata,
             Column('x', Integer, primary_key=True)
         )
         t1.create()
@@ -756,8 +853,10 @@ class SequenceExecTest(fixtures.TestBase):
         )
         self._assert_seq_result(r.inserted_primary_key[0])
 
+
 class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
     __requires__ = ('sequences',)
+    __backend__ = True
 
     @testing.fails_on('firebird', 'no FB support for start/increment')
     def test_start_increment(self):
@@ -772,12 +871,11 @@ class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
                 ]
                 start = seq.start or 1
                 inc = seq.increment or 1
-                assert values == list(xrange(start, start + inc * 3, inc))
+                assert values == list(range(start, start + inc * 3, inc))
 
             finally:
                 seq.drop(testing.db)
 
-
     def _has_sequence(self, name):
         return testing.db.dialect.has_sequence(testing.db, name)
 
@@ -785,15 +883,9 @@ class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         """test dialect renders the "nextval" construct,
         whether or not "optional" is set """
 
-        for s in (
-                Sequence("my_seq"),
-                Sequence("my_seq", optional=True)):
-            assert str(s.next_value().
-                    compile(dialect=testing.db.dialect)) in (
-                "nextval('my_seq')",
-                "gen_id(my_seq, 1)",
-                "my_seq.nextval",
-            )
+        for s in (Sequence("my_seq"), Sequence("my_seq", optional=True)):
+            assert str(s.next_value().compile(dialect=testing.db.dialect)) in (
+                "nextval('my_seq')", "gen_id(my_seq, 1)", "my_seq.nextval",)
 
     def test_nextval_unsupported(self):
         """test next_value() used on non-sequence platform
@@ -819,7 +911,7 @@ class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
 
     def test_checkfirst_metadata(self):
         m = MetaData()
-        s = Sequence("my_sequence", metadata=m)
+        Sequence("my_sequence", metadata=m)
         m.create_all(testing.db, checkfirst=False)
         assert self._has_sequence('my_sequence')
         m.create_all(testing.db, checkfirst=True)
@@ -841,14 +933,14 @@ class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
     @testing.provide_metadata
     def test_table_overrides_metadata_create(self):
         metadata = self.metadata
-        s1 = Sequence("s1", metadata=metadata)
+        Sequence("s1", metadata=metadata)
         s2 = Sequence("s2", metadata=metadata)
         s3 = Sequence("s3")
-        t = Table('t', metadata,
-                    Column('c', Integer, s3, primary_key=True))
+        t = Table(
+            't', metadata,
+            Column('c', Integer, s3, primary_key=True))
         assert s3.metadata is metadata
 
-
         t.create(testing.db, checkfirst=True)
         s3.drop(testing.db)
 
@@ -868,25 +960,32 @@ class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         assert not self._has_sequence('s1')
         assert not self._has_sequence('s2')
 
+cartitems = sometable = metadata = None
+
 
 class TableBoundSequenceTest(fixtures.TestBase):
     __requires__ = ('sequences',)
+    __backend__ = True
 
     @classmethod
     def setup_class(cls):
         global cartitems, sometable, metadata
         metadata = MetaData(testing.db)
-        cartitems = Table("cartitems", metadata,
-            Column("cart_id", Integer, Sequence('cart_id_seq'), primary_key=True),
+        cartitems = Table(
+            "cartitems", metadata,
+            Column(
+                "cart_id", Integer, Sequence('cart_id_seq'), primary_key=True),
             Column("description", String(40)),
             Column("createdate", sa.DateTime())
         )
-        sometable = Table( 'Manager', metadata,
-               Column('obj_id', Integer, Sequence('obj_id_seq'), ),
-               Column('name', String(128)),
-               Column('id', Integer, Sequence('Manager_id_seq', optional=True),
-                      primary_key=True),
-           )
+        sometable = Table(
+            'Manager', metadata,
+            Column('obj_id', Integer, Sequence('obj_id_seq')),
+            Column('name', String(128)),
+            Column(
+                'id', Integer, Sequence('Manager_id_seq', optional=True),
+                primary_key=True),
+        )
 
         metadata.create_all()
 
@@ -912,7 +1011,7 @@ class TableBoundSequenceTest(fixtures.TestBase):
     def test_seq_nonpk(self):
         """test sequences fire off as defaults on non-pk columns"""
 
-        engine = engines.testing_engine(options={'implicit_returning':False})
+        engine = engines.testing_engine(options={'implicit_returning': False})
         result = engine.execute(sometable.insert(), name="somename")
 
         assert set(result.postfetch_cols()) == set([sometable.c.obj_id])
@@ -921,8 +1020,8 @@ class TableBoundSequenceTest(fixtures.TestBase):
         assert set(result.postfetch_cols()) == set([sometable.c.obj_id])
 
         sometable.insert().execute(
-            {'name':'name3'},
-            {'name':'name4'})
+            {'name': 'name3'},
+            {'name': 'name4'})
         eq_(sometable.select().order_by(sometable.c.id).execute().fetchall(),
             [(1, "somename", 1),
              (2, "someother", 2),
@@ -931,17 +1030,20 @@ class TableBoundSequenceTest(fixtures.TestBase):
 
 
 class SpecialTypePKTest(fixtures.TestBase):
+
     """test process_result_value in conjunction with primary key columns.
 
-    Also tests that "autoincrement" checks are against column.type._type_affinity,
-    rather than the class of "type" itself.
+    Also tests that "autoincrement" checks are against
+    column.type._type_affinity, rather than the class of "type" itself.
 
     """
+    __backend__ = True
 
     @classmethod
     def setup_class(cls):
         class MyInteger(TypeDecorator):
             impl = Integer
+
             def process_bind_param(self, value, dialect):
                 if value is None:
                     return None
@@ -961,7 +1063,8 @@ class SpecialTypePKTest(fixtures.TestBase):
         kw['primary_key'] = True
         if kw.get('autoincrement', True):
             kw['test_needs_autoincrement'] = True
-        t = Table('x', metadata,
+        t = Table(
+            'x', metadata,
             Column('y', self.MyInteger, *arg, **kw),
             Column('data', Integer),
             implicit_returning=implicit_returning
@@ -971,7 +1074,9 @@ class SpecialTypePKTest(fixtures.TestBase):
         r = t.insert().values(data=5).execute()
 
         # we don't pre-fetch 'server_default'.
-        if 'server_default' in kw and (not testing.db.dialect.implicit_returning or not implicit_returning):
+        if 'server_default' in kw and (
+                not testing.db.dialect.implicit_returning or
+                not implicit_returning):
             eq_(r.inserted_primary_key, [None])
         else:
             eq_(r.inserted_primary_key, ['INT_1'])
@@ -988,7 +1093,8 @@ class SpecialTypePKTest(fixtures.TestBase):
         self._run_test()
 
     def test_literal_default_label(self):
-        self._run_test(default=literal("INT_1", type_=self.MyInteger).label('foo'))
+        self._run_test(
+            default=literal("INT_1", type_=self.MyInteger).label('foo'))
 
     def test_literal_default_no_label(self):
         self._run_test(default=literal("INT_1", type_=self.MyInteger))
@@ -1003,7 +1109,7 @@ class SpecialTypePKTest(fixtures.TestBase):
         self._run_test(server_default='1', autoincrement=False)
 
     def test_clause(self):
-        stmt = select([literal("INT_1", type_=self.MyInteger)]).as_scalar()
+        stmt = select([cast("INT_1", type_=self.MyInteger)]).as_scalar()
         self._run_test(default=stmt)
 
     @testing.requires.returning
@@ -1014,7 +1120,10 @@ class SpecialTypePKTest(fixtures.TestBase):
     def test_server_default_no_implicit_returning(self):
         self._run_test(server_default='1', autoincrement=False)
 
+
 class ServerDefaultsOnPKTest(fixtures.TestBase):
+    __backend__ = True
+
     @testing.provide_metadata
     def test_string_default_none_on_insert(self):
         """Test that without implicit returning, we return None for
@@ -1027,11 +1136,13 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
         """
 
         metadata = self.metadata
-        t = Table('x', metadata,
-                Column('y', String(10), server_default='key_one', primary_key=True),
-                Column('data', String(10)),
-                implicit_returning=False
-                )
+        t = Table(
+            'x', metadata,
+            Column(
+                'y', String(10), server_default='key_one', primary_key=True),
+            Column('data', String(10)),
+            implicit_returning=False
+        )
         metadata.create_all()
         r = t.insert().execute(data='data')
         eq_(r.inserted_primary_key, [None])
@@ -1043,13 +1154,16 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
     @testing.requires.returning
     @testing.provide_metadata
     def test_string_default_on_insert_with_returning(self):
-        """With implicit_returning, we get a string PK default back no problem."""
+        """With implicit_returning, we get a string PK default back no
+        problem."""
 
         metadata = self.metadata
-        t = Table('x', metadata,
-                Column('y', String(10), server_default='key_one', primary_key=True),
-                Column('data', String(10))
-                )
+        t = Table(
+            'x', metadata,
+            Column(
+                'y', String(10), server_default='key_one', primary_key=True),
+            Column('data', String(10))
+        )
         metadata.create_all()
         r = t.insert().execute(data='data')
         eq_(r.inserted_primary_key, ['key_one'])
@@ -1061,12 +1175,12 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
     @testing.provide_metadata
     def test_int_default_none_on_insert(self):
         metadata = self.metadata
-        t = Table('x', metadata,
-                Column('y', Integer,
-                        server_default='5', primary_key=True),
-                Column('data', String(10)),
-                implicit_returning=False
-                )
+        t = Table(
+            'x', metadata,
+            Column('y', Integer, server_default='5', primary_key=True),
+            Column('data', String(10)),
+            implicit_returning=False
+        )
         assert t._autoincrement_column is None
         metadata.create_all()
         r = t.insert().execute(data='data')
@@ -1081,15 +1195,16 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
                 t.select().execute().fetchall(),
                 [(5, 'data')]
             )
+
     @testing.provide_metadata
     def test_autoincrement_reflected_from_server_default(self):
         metadata = self.metadata
-        t = Table('x', metadata,
-                Column('y', Integer,
-                        server_default='5', primary_key=True),
-                Column('data', String(10)),
-                implicit_returning=False
-                )
+        t = Table(
+            'x', metadata,
+            Column('y', Integer, server_default='5', primary_key=True),
+            Column('data', String(10)),
+            implicit_returning=False
+        )
         assert t._autoincrement_column is None
         metadata.create_all()
 
@@ -1100,12 +1215,12 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
     @testing.provide_metadata
     def test_int_default_none_on_insert_reflected(self):
         metadata = self.metadata
-        t = Table('x', metadata,
-                Column('y', Integer,
-                        server_default='5', primary_key=True),
-                Column('data', String(10)),
-                implicit_returning=False
-                )
+        Table(
+            'x', metadata,
+            Column('y', Integer, server_default='5', primary_key=True),
+            Column('data', String(10)),
+            implicit_returning=False
+        )
         metadata.create_all()
 
         m2 = MetaData(metadata.bind)
@@ -1128,11 +1243,11 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
     @testing.provide_metadata
     def test_int_default_on_insert_with_returning(self):
         metadata = self.metadata
-        t = Table('x', metadata,
-                Column('y', Integer,
-                        server_default='5', primary_key=True),
-                Column('data', String(10))
-                )
+        t = Table(
+            'x', metadata,
+            Column('y', Integer, server_default='5', primary_key=True),
+            Column('data', String(10))
+        )
 
         metadata.create_all()
         r = t.insert().execute(data='data')
@@ -1142,25 +1257,19 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
             [(5, 'data')]
         )
 
+
 class UnicodeDefaultsTest(fixtures.TestBase):
+    __backend__ = True
+
     def test_no_default(self):
-        c = Column(Unicode(32))
+        Column(Unicode(32))
 
     def test_unicode_default(self):
-        # Py3K
-        #default = 'foo'
-        # Py2K
-        default = u'foo'
-        # end Py2K
-        c = Column(Unicode(32), default=default)
-
+        default = u('foo')
+        Column(Unicode(32), default=default)
 
     def test_nonunicode_default(self):
-        # Py3K
-        #default = b'foo'
-        # Py2K
-        default = 'foo'
-        # end Py2K
+        default = b('foo')
         assert_raises_message(
             sa.exc.SAWarning,
             "Unicode column received non-unicode default value.",
